import { Ref, UnwrapRef, InjectionKey, UnwrapNestedRefs as UnwrapNestedRefs$1, ComponentPropsOptions, ExtractPropTypes, Plugin } from 'vue-demi';
import { AnyFunction, DropParameters } from '@winter-love/utils';
import { CustomInspectorNode, StateBase, App } from '@vue/devtools-api';

declare type OneAndAnyFunc<T, A extends any[] = any[], R = any> = (one: T, ...args: A) => R;
declare type UnwrapNestedRefs<T> = T extends Ref ? T : UnwrapRef<T>;
declare type DropFunctionObject<T extends Record<string, AnyFunction>, S = any> = {
    [P in keyof T]: (...args: DropParameters<T[P], S>) => ReturnType<T[P]>;
};

declare const STORE_TREE_KEY: unique symbol;
declare const STORE_CONTEXT: InjectionKey<StoreManager>;
declare const STORE_LOCAL_CONTEXT: InjectionKey<StoreManager>;

interface StoreManagerItem<T extends Record<string, any> = Record<string, any>> {
    props: UnwrapNestedRefs$1<Record<string, any>>;
    store: UnwrapNestedRefs$1<T>;
}
interface StoreTreeInfo {
    kind: string;
}
interface StoreTree {
    [key: string]: any;
    [STORE_TREE_KEY]?: StoreTreeInfo;
}
declare const useStoreManager: () => Readonly<{
    get(name: string): any;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem): void;
    readonly storeTree: UnwrapNestedRefs$1<Record<string, any>>;
    update(source: Record<string, any>): any;
}>;
declare type StoreManager = Readonly<{
    get(name: string): any;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem): void;
    readonly storeTree: UnwrapNestedRefs$1<Record<string, any>>;
    update(source: Record<string, any>): any;
}>;
declare const createManager: (info?: StoreTreeInfo | undefined) => StoreManager;

declare const createTree: (stateTree: UnwrapNestedRefs$1<any>, index?: number) => CustomInspectorNode[];
declare const createTreeMultiple: (...stateTrees: UnwrapNestedRefs$1<any>[]) => CustomInspectorNode[];
declare const createInspect: (name: any, state: UnwrapNestedRefs$1<any>) => Record<string, StateBase[]>;
declare const createStoreDevTool: (app: App, stateTree: UnwrapNestedRefs$1<any> | UnwrapNestedRefs$1<any>[]) => void;

declare type EmptyObject = {};
interface UseStore<T, P> {
    (props?: ObjectWithRef<P>, options?: UseStoreOptions): Store<T>;
    readonly local: (props?: ObjectWithRef<P>, options?: Omit<UseStoreOptions, 'local'>) => Store<T>;
}
declare type ObjectWithRef<T extends Record<string, any>> = {
    [P in keyof T]: Ref<T[P]> | T[P];
};
declare type Store<T> = UnwrapNestedRefs$1<T> & {
    readonly __store?: never;
};
interface UseStoreOptions {
    /**
     * use as a normal composition api function
     */
    local?: boolean;
    /**
     * reset saved store
     */
    reset?: boolean;
}
interface SetupContext<Root> {
    readonly root: UnwrapNestedRefs$1<Root>;
}
declare type Setup<T extends Record<string, any>, P extends Record<string, any>, Root extends Record<string, any> = Record<string, any>> = (props: UnwrapNestedRefs$1<P>, context: SetupContext<Root>) => T;
interface CreateStoreOptions<T extends Record<string, any>, P extends ComponentPropsOptions = ComponentPropsOptions> {
    /**
     * @experimental use at using time
     */
    local?: boolean;
    name: string;
    props?: P;
    setup: Setup<T, Readonly<ExtractPropTypes<P>>>;
}
interface CreateSetupArgs<T extends Record<string, any>, P extends Record<string, any> = EmptyObject> {
    props?: P;
    setup: Setup<T, P>;
    storeManager: StoreManager;
}
interface CreateLocalStoreArgs<T extends Record<string, any>, P extends Record<string, any> = EmptyObject> extends CreateSetupArgs<T, P> {
    name: string;
}
interface CreateGlobalStoreArgs<T extends Record<string, any>, P extends Record<string, any> = EmptyObject> {
    name: string;
    options: Omit<UseStoreOptions, 'local'>;
    props?: P;
    setup: Setup<T, P>;
    storeManager: StoreManager;
}
declare const useLocalManager: () => StoreManager | undefined;
declare function defineStore<T extends Record<string, any>, P extends ComponentPropsOptions = ComponentPropsOptions>(options: CreateStoreOptions<T, P>): UseStore<T, Readonly<ExtractPropTypes<P>>>;
declare function defineStore<T extends Record<string, any>>(name: string, setup: Setup<T, EmptyObject>): UseStore<T, EmptyObject>;
declare function createStore<T extends Record<string, any>, P extends ComponentPropsOptions = ComponentPropsOptions>(options: CreateStoreOptions<T, P>): UseStore<T, Readonly<ExtractPropTypes<P>>>;
declare function createStore<T extends Record<string, any>>(name: string, setup: Setup<T, EmptyObject>): UseStore<T, EmptyObject>;
/**
 * @deprecated
 * @param options
 */
declare function useStore<T extends Record<string, any>, P extends Record<string, any>>(options: CreateStoreOptions<T, P>): any;
declare function useStore<T extends Record<string, any>, P extends Record<string, any>>(name: string, setup: Setup<T, P>): any;

interface Vare {
    install: Plugin;
    localManager: StoreManager;
    manager: StoreManager;
}
declare const createVare: () => Vare;
declare const createVareStore: () => Vare;
declare const provideStoreManager: (manager?: Readonly<{
    get(name: string): any;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem<Record<string, any>>): void;
    readonly storeTree: {
        [x: string]: any;
    };
    update(source: Record<string, any>): any;
}> | undefined, localManager?: Readonly<{
    get(name: string): any;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem<Record<string, any>>): void;
    readonly storeTree: {
        [x: string]: any;
    };
    update(source: Record<string, any>): any;
}> | undefined) => void;

export { CreateGlobalStoreArgs, CreateLocalStoreArgs, CreateSetupArgs, CreateStoreOptions, DropFunctionObject, EmptyObject, ObjectWithRef, OneAndAnyFunc, STORE_CONTEXT, STORE_LOCAL_CONTEXT, STORE_TREE_KEY, Setup, SetupContext, Store, StoreManager, StoreManagerItem, StoreTree, StoreTreeInfo, UnwrapNestedRefs, UseStore, UseStoreOptions, Vare, createInspect, createManager, createStore, createStoreDevTool, createTree, createTreeMultiple, createVare, createVareStore, defineStore, provideStoreManager, useLocalManager, useStore, useStoreManager };
