var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));

// src/store/index.ts
import { provide } from "vue-demi";

// src/store/manager.ts
import { inject, reactive } from "vue-demi";

// src/store/shallow-update.ts
var shallowUpdate = (target, source) => {
  Object.keys(source).forEach((key) => {
    target[key] = source[key];
  });
};
var oneDepthUpdate = (target, source) => {
  Object.keys(source).forEach((key) => {
    const item = source[key];
    const targetItem = target[key];
    if (typeof item === "object" && typeof targetItem === "object") {
      shallowUpdate(targetItem, item);
      return;
    }
    target[key] = source[key];
  });
};

// src/store/symbols.ts
var STORE_TREE_KEY = Symbol("store-tree");
var STORE_CONTEXT = Symbol("store");
var STORE_LOCAL_CONTEXT = Symbol("store-local");

// src/store/manager.ts
var useStoreManager = () => {
  var _a;
  return (_a = inject(STORE_CONTEXT)) != null ? _a : createManager();
};
var createManager = (info) => {
  const storeTree = reactive({
    [STORE_TREE_KEY]: info
  });
  const storePropsMap = /* @__PURE__ */ new Map();
  const set = (name, item) => {
    storeTree[name] = item.store;
    storePropsMap.set(name, item.props);
  };
  const remove = (name) => {
    storeTree[name] = void 0;
    storePropsMap.delete(name);
  };
  const get = (name) => {
    var _a;
    return {
      props: (_a = storePropsMap.get(name)) != null ? _a : {},
      store: storeTree[name]
    };
  };
  const update = (source) => {
    oneDepthUpdate(storeTree, source);
  };
  return {
    get,
    remove,
    set,
    storeTree,
    update
  };
};

// src/store/dev-tool.ts
import { setupDevtoolsPlugin } from "@vue/devtools-api";
import { drop, parseJson, toArray } from "@winter-love/utils";
import { watch } from "vue-demi";
var tagThemes = {
  "local-state": {
    backgroundColor: 7580670,
    label: "local state",
    textColor: 0
  },
  state: {
    backgroundColor: 394748,
    label: "state",
    textColor: 0
  },
  unknown: {
    backgroundColor: 16758528,
    label: "unknown",
    textColor: 0
  }
};
var createTree = (stateTree, index = 0) => {
  const info = stateTree[STORE_TREE_KEY];
  return Object.keys(stateTree).map((key) => {
    var _a;
    const itemInfo = parseJson(key, key);
    const tag = typeof itemInfo === "string" ? [] : [
      {
        backgroundColor: 10066329,
        label: `for: ${itemInfo.componentName}`,
        textColor: 0
      },
      {
        backgroundColor: 10066329,
        label: itemInfo.uid,
        textColor: 0
      }
    ];
    return {
      id: JSON.stringify({
        index,
        key
      }),
      label: typeof itemInfo === "string" ? itemInfo : itemInfo.name,
      tags: [
        __spreadValues({}, tagThemes[(_a = info == null ? void 0 : info.kind) != null ? _a : "unknown"]),
        ...tag
      ]
    };
  });
};
var createTreeMultiple = (...stateTrees) => {
  return stateTrees.flatMap((stateTree, index) => {
    return createTree(stateTree, index);
  });
};
var createInspect = (name, state) => {
  const itemInfo = parseJson(name, name);
  return {
    state: [
      {
        editable: true,
        key: typeof itemInfo === "string" ? itemInfo : itemInfo.name,
        objectType: "reactive",
        value: state
      }
    ]
  };
};
var createStoreDevTool = (app, stateTree) => {
  const inspectId = "vare-inspect";
  const timeLineId = "vare-event";
  const label = "vare-store";
  const stateTrees = toArray(stateTree);
  setupDevtoolsPlugin({
    app,
    id: "vare-store",
    label: "vare-store",
    packageName: "vare"
  }, (api) => {
    api.addInspector({
      icon: "mediation",
      id: inspectId,
      label: `${label} Structure`
    });
    api.addTimelineLayer({
      color: 15764809,
      id: timeLineId,
      label: "Vare Changing"
    });
    api.on.getInspectorTree((payload) => {
      if (payload.app !== app || payload.inspectorId !== inspectId) {
        return;
      }
      payload.rootNodes = createTreeMultiple(...stateTrees);
    });
    api.on.getInspectorState((payload) => {
      if (payload.app !== app || payload.inspectorId !== inspectId) {
        return;
      }
      const name = payload.nodeId;
      const { key, index } = parseJson(name, { index: 0, key: name });
      const state = stateTrees[index][key];
      if (state) {
        payload.state = createInspect(key, state);
      }
    });
    api.on.editInspectorState((payload) => {
      if (payload.app !== app || payload.inspectorId !== inspectId) {
        return;
      }
      const name = payload.nodeId;
      const { key, index } = parseJson(name, { index: 0, key: name });
      const state = stateTrees[index][key];
      if (state) {
        const path = drop(1)(payload.path);
        const { value } = payload.state;
        payload.set(state, path, value);
      }
    });
    Object.keys(stateTree).forEach((name) => {
      watch(stateTree[name], () => {
        api.addTimelineEvent({
          event: {
            data: typeof stateTree[name] === "object" ? __spreadValues({}, stateTree[name]) : stateTree[name],
            time: Date.now()
          },
          layerId: timeLineId
        });
      });
    });
  });
};

// src/store/store.ts
import {
  getCurrentInstance,
  inject as inject2,
  onScopeDispose,
  reactive as reactive2,
  watch as watch2
} from "vue-demi";
import { freeze } from "@winter-love/utils";

// src/store/props-validator.ts
var getTypeName = (value) => {
  switch (value) {
    case String:
      return "string";
    case Number:
      return "number";
    case Object:
      return "object";
    case Boolean:
      return "boolean";
    default:
      return "any";
  }
};
var propsValidator = (props, propsOptions) => {
  if (!propsOptions) {
    return true;
  }
  return Object.entries(props).every(([key, value]) => {
    const option = propsOptions[key];
    if (!option) {
      return true;
    }
    const { validator, type } = option;
    if (typeof validator === "function") {
      return validator(value);
    }
    if (type) {
      const typeName = getTypeName(type);
      if (typeName === "any") {
        return true;
      }
      return typeof value === typeName;
    }
    return true;
  });
};

// src/store/create-uuid.ts
var createUuid = () => {
  let count = 0;
  return () => {
    count += 1;
    return count;
  };
};

// src/store/store.ts
var getUuid = createUuid();
var createSetup = (args) => {
  const { setup, props, storeManager } = args;
  const _props = reactive2(props != null ? props : {});
  return {
    props: _props,
    store: reactive2(setup(_props, {
      root: storeManager.storeTree
    }))
  };
};
var createLocalStore = (args) => {
  var _a, _b, _c;
  const { name, storeManager, setup, props } = args;
  const item = createSetup({
    props,
    setup,
    storeManager
  });
  const instance = getCurrentInstance();
  const uid = instance ? instance.uid : getUuid();
  const componentName = (_c = (_b = (_a = instance == null ? void 0 : instance.ctx) == null ? void 0 : _a.$options) == null ? void 0 : _b.name) != null ? _c : "Undefined Name";
  const uuidName = process.env.NODE_ENV === "production" ? `${uid}` : JSON.stringify({
    componentName,
    name,
    uid
  });
  const localStoreManager = useLocalManager();
  if (localStoreManager) {
    localStoreManager.set(uuidName, item);
  }
  onScopeDispose(() => {
    localStoreManager == null ? void 0 : localStoreManager.remove(uuidName);
  });
  return item.store;
};
var createGlobalStore = (args) => {
  const { name, options, props, storeManager, setup } = args;
  const { reset } = options;
  const savedState = storeManager.get(name);
  if (savedState) {
    const savedProps = savedState.props;
    const currentProps = reactive2(props != null ? props : {});
    watch2(currentProps, (props2) => {
      Object.keys(props2).forEach((key) => {
        savedProps[key] = currentProps[key];
      });
    });
  }
  const state = !reset && (savedState == null ? void 0 : savedState.store) ? savedState : createSetup({
    props,
    setup,
    storeManager
  });
  storeManager.set(name, state);
  return state.store;
};
var useLocalManager = () => {
  return inject2(STORE_LOCAL_CONTEXT);
};
var getOptions = (arg1, arg2) => {
  if (typeof arg1 === "string" && typeof arg2 !== "undefined") {
    return {
      name: arg1,
      setup: arg2
    };
  }
  if (typeof arg1 === "object" && typeof arg2 === "undefined") {
    return arg1;
  }
  throw new Error("params error");
};
function defineStore(arg1, arg2) {
  const {
    name,
    setup,
    local,
    props: propsOptions
  } = getOptions(arg1, arg2);
  const hook = (props, options = {}) => {
    const { local: innerLocal = local } = options;
    if (process.env.NODE_ENV !== "production") {
      const result = propsValidator(props != null ? props : {}, propsOptions);
      if (result !== true) {
        console.warn(result);
      }
    }
    const storeManager = useStoreManager();
    if (innerLocal) {
      return createLocalStore({ name, props, setup, storeManager });
    }
    return createGlobalStore({
      name,
      options,
      props,
      setup,
      storeManager
    });
  };
  return freeze(Object.assign(hook, {
    local: (props, options = {}) => hook(props, __spreadProps(__spreadValues({}, options), { local: true }))
  }));
}
function createStore(arg1, arg2) {
  return defineStore(arg1, arg2);
}
function useStore(arg1, arg2) {
  return createStore(arg1, arg2)();
}

// src/store/index.ts
var createVare = () => {
  const manager = createManager({
    kind: "state"
  });
  const localManager = createManager({
    kind: "local-state"
  });
  return {
    install: (app) => {
      app.provide(STORE_CONTEXT, manager);
      app.provide(STORE_LOCAL_CONTEXT, localManager);
    },
    localManager,
    manager
  };
};
var createVareStore = createVare;
var provideStoreManager = (manager, localManager) => {
  provide(STORE_CONTEXT, manager != null ? manager : createManager());
  if (process.env.NODE_ENV !== "production") {
    provide(STORE_LOCAL_CONTEXT, localManager);
  }
};
export {
  STORE_CONTEXT,
  STORE_LOCAL_CONTEXT,
  STORE_TREE_KEY,
  createInspect,
  createManager,
  createStore,
  createStoreDevTool,
  createTree,
  createTreeMultiple,
  createVare,
  createVareStore,
  defineStore,
  provideStoreManager,
  useLocalManager,
  useStore,
  useStoreManager
};
//# sourceMappingURL=index.mjs.map